package com.idea.customerManagement.service;

import cn.hutool.core.math.Money;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.MnpBuildingMapper;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.dto.IntentionalDepositDto;
import com.idea.customerManagement.dto.RoomSelectionInfoDto;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoExtendMapper;
import com.idea.customerManagement.mapper.RoomSelectionInfoMapper;
import com.idea.customerManagement.model.*;
import com.idea.invoice.model.ReceiptManage;
import com.idea.invoice.model.ReceiptManageExample;
import com.idea.invoice.service.ReceiptManageService;
import com.idea.invoice.util.InvoiceConstant;
import com.idea.invoice.util.InvoiceUtil;
import com.idea.paymentManagement.mapper.PayLogMapper;
import com.idea.paymentManagement.model.PayLog;
import com.idea.paymentManagement.model.PayLogExample;
import com.idea.util.DateUtils;
import com.idea.util.MoneyUtils;
import com.idea.util.ReplaceWord;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.*;

@Slf4j
@Service
public class RoomSelectionInfoService implements BaseService<RoomSelectionInfo, RoomSelectionInfoExample> {

    @Autowired
    private RoomSelectionInfoMapper modelMapper;
    @Autowired
    private RoomSelectionInfoExtendMapper extendMapper;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private ParkRoomExtendMapper roomExtendMapper;
    @Autowired
    private ContractManageService contractManageService;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private ParkRoomMapper parkRoomMapper;
    @Autowired
    private PayLogMapper payLogMapper;
    @Autowired
    private ReceiptManageService receiptManageService;
    @Autowired
    private SysDictService sysDictService;


    @Override
    public int deleteByPrimaryKey(String s) {
        return modelMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(RoomSelectionInfo roomSelectionInfo) {
        roomSelectionInfo.setId(IdUtil.simpleUUID());
        return modelMapper.insertSelective(roomSelectionInfo);
    }

    public int insertWithoutId(RoomSelectionInfo roomSelectionInfo) {
        return modelMapper.insertSelective(roomSelectionInfo);
    }

    @Override
    public RoomSelectionInfo selectByPrimaryKey(String s) {
        return modelMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(RoomSelectionInfo roomSelectionInfo) {
        return modelMapper.updateByPrimaryKeySelective(roomSelectionInfo);
    }

    @Override
    public int updateByExampleSelective(RoomSelectionInfo roomSelectionInfo, RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.updateByExampleSelective(roomSelectionInfo, roomSelectionInfoExample);
    }

    @Override
    public int updateByExample(RoomSelectionInfo roomSelectionInfo, RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.updateByExample(roomSelectionInfo, roomSelectionInfoExample);
    }

    @Override
    public List<RoomSelectionInfo> selectByExample(RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.selectByExample(roomSelectionInfoExample);
    }

    @Override
    public long countByExample(RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.countByExample(roomSelectionInfoExample);
    }

    @Override
    public int deleteByExample(RoomSelectionInfoExample roomSelectionInfoExample) {
        return modelMapper.deleteByExample(roomSelectionInfoExample);
    }

    /**
     * 选房信息列表
     *
     * @param customerManagementId
     * @return
     */
    public List<RoomSelectionInfoDto> listAll(String customerManagementId) {
        List<RoomSelectionInfoDto> result = extendMapper.selectHouseListByCustomerId(customerManagementId);
        return result;
    }

    /**
     * 提交选房
     *
     * @param houseIds
     * @param customerManagementId
     * @return
     */
    public int submit(String houseIds, String customerManagementId, String userId) {

        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(customerManagementId);
        customerManagement.setStatus(2);
        customerManagement.setHouseId(houseIds);
        String[] split = houseIds.split(",");
        StringBuilder builder = new StringBuilder();
        // 先删除
        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<RoomSelectionInfo> list = selectByExample(example);
        if (CollectionUtils.isNotEmpty(list)) {
            for (RoomSelectionInfo info : list) {
                deleteByPrimaryKey(info.getId());
                // 还原房间选房状态
                ParkRoom parkRoom = new ParkRoom();
                parkRoom.setId(info.getHouseId());
                parkRoom.setIsChoose(0);
                parkRoomMapper.updateByPrimaryKeySelective(parkRoom);
            }
        }
        // 再新增
        for (String s : split) {
            // 拼接房屋名称
            ParkRoomResponse response = roomExtendMapper.getFullName(s);
            builder.append(response.getBuildName()).append("-").append(response.getRoomNo()).append(",");
            // 新增选房信息
            RoomSelectionInfo roomSelectionInfo = new RoomSelectionInfo();
            roomSelectionInfo.setCustomerManagementId(customerManagementId);
            roomSelectionInfo.setHouseId(response.getId());
            roomSelectionInfo.setBuildId(response.getBuildId());
            roomSelectionInfo.setDiscId(response.getDiscId());
            roomSelectionInfo.setGroupId(response.getGroupId());
            roomSelectionInfo.setCreatedAt(new Date());
            roomSelectionInfo.setCreatedId(userId == null ? ShiroUtils.getUserId() : userId);
            roomSelectionInfo.setReceivableMoney(response.getSubscriptionFunds());
            roomSelectionInfo.setStatus(1);
            insertSelective(roomSelectionInfo);

            ParkRoom parkRoom = new ParkRoom();
            parkRoom.setId(response.getId());
            parkRoom.setIsChoose(1);
            parkRoomMapper.updateByPrimaryKeySelective(parkRoom);

        }
        if (builder.length() > 0) {
            builder.deleteCharAt(builder.length() - 1);
        }
        customerManagement.setHouseName(builder.toString());
        customerManagementMapper.updateByPrimaryKeySelective(customerManagement);
        return 1;
    }

    /**
     * 获取收据流水号（定金编号）
     *
     * @param houseId
     * @return
     */
    public String getSerialNumber(String houseId) {
        ParkRoomResponse response = roomExtendMapper.getFullName(houseId);
        String groupId = response.getGroupId();
        // 查找这个小区已收取认购金的 数据量
        RoomSelectionInfoExample example = new RoomSelectionInfoExample();
        example.createCriteria().andGroupIdEqualTo(groupId).andStatusEqualTo(2);
        Long count = countByExample(example);
        String number = "";
        if (count.doubleValue() == 0) {
            number = "00001";
        } else {
            count++;
            number = String.format("%05d", count);
        }
        String serialNumber = response.getGroupName() + response.getDiscName() + "-DJ" + number;
        return serialNumber;
    }

    /**
     * 认购金登记
     *
     * @param model
     * @return
     */
    @Transactional
    public int add(RoomSelectionInfoDto model,String userId) {

        if(StringUtils.isEmpty(userId)){
            userId = ShiroUtils.getUserId();
        }
        model.setUpdatedAt(new Date());
        model.setUpdatedId(userId);
        model.setStatus(2);
        model.setCollectionId(userId);
        model.setCollectionTime(new Date());
        int result = updateByPrimaryKeySelective(model);
        // 生成合同
        String contractId = IdUtil.simpleUUID();
        contractManageService.addContract(model, contractId,userId);
        // 修改房间已收状态 已认购
        ParkRoom parkRoom = new ParkRoom();
        parkRoom.setId(model.getHouseId());
        parkRoom.setSoldStatus(2);
        parkRoomMapper.updateByPrimaryKeySelective(parkRoom);
        // 新增一条定金收款记录
        PayLog payLog = new PayLog();
        payLog.setId(IdUtil.simpleUUID());
        payLog.setBusinessId(payLog.getId());
        payLog.setHouseId(model.getHouseId());
        payLog.setContractId(contractId);
        payLog.setCustomerManagementId(model.getCustomerManagementId());
        // 收款类型 预收款
        payLog.setPayType(1);
        // 款项内容 定金
        payLog.setContentType(1);
        payLog.setPayMoney(model.getReceivedAmount());
        payLog.setPayTime(new Date());
        payLog.setPaymentMethod(model.getPaymentMethod());
        payLog.setStatus(0);
        payLog.setCreatedAt(new Date());
        payLog.setCreatedBy(userId);
        payLog.setSerialNumber(model.getSerialNumber());
        payLogMapper.insertSelective(payLog);
        // 票据管理 新增一条记录
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(model.getCustomerManagementId());
        ReceiptManage receiptManage = new ReceiptManage();
        receiptManage.setBatchNumber(customerManagement.getBatchNumber());
        receiptManage.setRoomSelectionNumber(customerManagement.getRoomSelectionNumber());
        receiptManage.setBuyerName(customerManagement.getBuyerName());
        receiptManage.setCustomerManagementId(model.getCustomerManagementId());
        receiptManage.setBusinessId(model.getId());
        // 票据类型 定金
        receiptManage.setReceiptType("2");
        receiptManage.setReceiptMoney(model.getReceivedAmount());
        receiptManage.setReceiptNumber(model.getSerialNumber());
        receiptManageService.add(receiptManage,userId);
        return result;
    }

    /**
     * 认购金修改
     *
     * @param model
     * @return
     */
    @Transactional
    public int edit(RoomSelectionInfoDto model,String userId) {

        if (StringUtils.isEmpty(userId)){
            userId = ShiroUtils.getUserId();
        }
        model.setUpdatedAt(new Date());
        model.setUpdatedId(userId);
        model.setCollectionId(userId);
        int result = updateByPrimaryKeySelective(model);

        // 收款记录 定金数值调整
        PayLogExample payLogExample = new PayLogExample();
        payLogExample.createCriteria().andBusinessIdEqualTo(model.getId());
        List<PayLog> payLogs = payLogMapper.selectByExample(payLogExample);
        if(CollectionUtils.isNotEmpty(payLogs)){
            PayLog payLog = payLogs.get(0);
            payLog.setPayMoney(model.getReceivedAmount());
            payLogMapper.updateByPrimaryKeySelective(payLog);
        }

        // 票据管理 定金数值调整
        ReceiptManageExample receiptManageExample = new ReceiptManageExample();
        receiptManageExample.createCriteria().andBusinessIdEqualTo(model.getId()).andReceiptTypeEqualTo("2");
        List<ReceiptManage> receiptManages = receiptManageService.selectByExample(receiptManageExample);
        if (CollectionUtils.isNotEmpty(receiptManages)) {
            ReceiptManage manage = receiptManages.get(0);
            manage.setReceiptMoney(model.getReceivedAmount());
            receiptManageService.updateByPrimaryKeySelective(manage);
        }

        return result;
    }

    /**
     * 查看详情
     *
     * @param id
     * @return
     */
    public RoomSelectionInfo getById(String id) {
        return selectByPrimaryKey(id);
    }

    /**
     * 房源确认单 下载
     *
     * @param model
     * @param request
     * @param response
     */
    public void downLoad(RoomSelectionInfoDto model, HttpServletRequest request, HttpServletResponse response) {

        String filePath = "/home/project/settleDown/tem/roomConfirmationSheet.docx";
        String fileName = "房源确认单.docx";
        String customerManagementId = model.getCustomerManagementId();
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        if (CollectionUtils.isNotEmpty(buyers)) {
            Buyer buyer = buyers.get(0);
            ParkRoomResponse roomResponse = roomExtendMapper.getFullName(model.getHouseId());
            String fullName = roomResponse.getGroupName() + roomResponse.getDiscName() + roomResponse.getBuildName()
                    + roomResponse.getRoomNo();
            String loginName = ShiroUtils.getLoginName();
            String roomChooseDate = DateUtils.parseDateToStr(DateUtils.DATE_CH, new Date());

            Map<String, Object> data = new HashMap<>();
            Map<String, Object> picData = new HashMap<>();
            Map<String, Object> sealData = new HashMap<>();
            Map<String, Object> settingData = new HashMap<>();

            data.put("${name}", buyer.getName());
            data.put("${identityCard}", buyer.getIdentityCard());
            data.put("${address}", fullName);
            data.put("${area}", roomResponse.getPredictionBuildArea() == null ? "" : roomResponse.getPredictionBuildArea());
            data.put("${roomNumber}", roomResponse.getRoomNumber() == null ? "" : roomResponse.getRoomNumber());
            data.put("${roomChooseDate}", roomChooseDate);
            data.put("${username}", loginName);
            ReplaceWord.operateWord(response, filePath, fileName, data, picData, sealData, settingData);
        }


    }


    /**
     * 定金列表
     *
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    public PageInfo<RoomSelectionInfoDto> listByModel(Tablepar tablepar, RoomSelectionInfoDto model, DateTrans dt) {
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<RoomSelectionInfoDto> list = extendMapper.listByModel(model);
        if(StringUtils.isNotEmpty(model.getDiscIds())){
            List<String> ids = Arrays.asList(model.getDiscIds().split(","));
            model.setDiscIdList(ids);
        }
        PageInfo<RoomSelectionInfoDto> pageInfo = new PageInfo<>(list);
        return pageInfo;
    }

    /**
     * 定金收据
     *
     * @param model
     * @param response
     */
    public void downLoadReceipt(RoomSelectionInfoDto model, HttpServletResponse response) {

        RoomSelectionInfo newRecord = selectByPrimaryKey(model.getId());
        String filePath = "/home/project/settleDown/tem/receipt.docx";
        String fileName = "定金收据.docx";
        String sealPath = "/home/project/settleDown/tem/seal.png";
        String customerManagementId = newRecord.getCustomerManagementId();
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(customerManagementId);
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        StringBuilder builder = new StringBuilder();
        if (CollectionUtils.isNotEmpty(buyers)) {
            for (Buyer buyer : buyers) {
                builder.append(buyer.getName()).append(",");
            }
            if (builder.length() > 0) {
                builder.deleteCharAt(builder.length() - 1);
            }
        }

        ParkRoomResponse roomResponse = roomExtendMapper.getFullName(newRecord.getHouseId());
        String fullName = roomResponse.getGroupName() + "-"
                + roomResponse.getDiscName() + "-"
                + roomResponse.getBuildName() + "-"
                + roomResponse.getRoomNo();

        String serialNumber = newRecord.getSerialNumber();
        String collectionTime = DateUtils.parseDateToStr(DateUtils.DATE_CH, newRecord.getCollectionTime());
        String content = "定金";
        String paymentMethod = "";
        String dictLabel = sysDictService.getDictLabel("COLLECTION_METHODS", newRecord.getPaymentMethod().toString());
        if (StringUtils.isNotEmpty(dictLabel)) {
            paymentMethod = dictLabel;
        }
        String receivedAmount = newRecord.getReceivedAmount().toString();
        String capital = MoneyUtils.amountConversion(receivedAmount);

        Map<String, Object> data = new HashMap<>();
        Map<String, Object> picData = new HashMap<>();
        Map<String, Object> sealData = new HashMap<>();
        Map<String, Object> settingData = new HashMap<>();
        Map<String, Object> settingInfoData;

        data.put("${serialNumber}", serialNumber);
        data.put("${name}", builder.toString());
        data.put("${collectionTime}", collectionTime);
        data.put("${content}", content);
        data.put("${paymentMethod}", paymentMethod);
        data.put("${fullName}", fullName);
        data.put("${capital}", capital);
        data.put("${receivedAmount}", receivedAmount);
        sealData.put("${seal}", sealPath);

        settingInfoData = new HashMap<>();
        settingInfoData.put("leftOffset", "50");
        settingInfoData.put("topOffset", "0");
        settingData.put("${seal}", settingInfoData);

        ReplaceWord.operateWord(response, filePath, fileName, data, picData, sealData, settingData);


    }


    public AjaxResult sendVoucher(RoomSelectionInfoDto model) {

        // 银行流水id
        String pkInformerId = model.getPkInformerId();
        RoomSelectionInfo roomSelectionInfo = selectByPrimaryKey(model.getId());
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(roomSelectionInfo.getCustomerManagementId());
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(customerManagement.getId());
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        Buyer buyer = buyers.get(0);

        JSONObject json = new JSONObject();
        JSONObject ar_gatherbill = new JSONObject();
        String srcsystemid = model.getId();
        String billdate = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, new Date());
        ar_gatherbill.put("srcsystemid", srcsystemid); //外系统标识,必选
        ar_gatherbill.put("pk_tradetype", InvoiceConstant.INVOICE_BUSINESS_F2_Cxx_06); //交易类型,必选
        ar_gatherbill.put("pk_org", InvoiceUtil.pk_org); //组织编码,必选
        ar_gatherbill.put("billdate", billdate); //单据日期
        ar_gatherbill.put("def2", srcsystemid); //外系统单据号，必填，
        json.put("ar_gatherbill", ar_gatherbill);

        JSONArray ar_gatheritem = new JSONArray();
        JSONObject ar_gatheritemJson = new JSONObject();
        ar_gatheritemJson.put("srcsystemid", srcsystemid); //外系统子表标识
        ar_gatheritemJson.put("objtype", "0"); //0=客户，1=供应商--判断customer和supplier是否必填，必选
        ar_gatheritemJson.put("customer", buyer.getIdentityCard()); //客商的纳税人识别号
        ar_gatheritemJson.put("money_cr", roomSelectionInfo.getReceivedAmount()); //金额，必选
        ar_gatheritemJson.put("taxrate", "10"); //税率,必选，数字——3/10/6````
        ar_gatheritemJson.put("pk_balatype", "0"); //结算方式,必选，看云文档结算方式
        ar_gatheritemJson.put("pk_recpaytype", "001"); //付款业务类型传默认值“001”
        ar_gatheritemJson.put("prepay", "0"); //收款性质应收款时传"0",收款性质预收款时传"1"
        ar_gatheritemJson.put("recaccount", InvoiceUtil.bank_number); //收款银行账号,必填
//        ar_gatheritemJson.put("invoiceno","fphm"); //发票号码，非必选
        ar_gatheritemJson.put("scomment", buyer.getName()); //客户名称，必选
        ar_gatheritemJson.put("pk_subjcode", InvoiceConstant.INVOICE_PAYMENT_TYPE_AJ001); //收款类型
        ar_gatheritemJson.put("checktype", InvoiceConstant.INVOICE_RECEIPT_TYPE_3); //票据类型，必选，看云文档票据类型
        ar_gatheritemJson.put("checkno", roomSelectionInfo.getSerialNumber()); //票据号，必选
        ar_gatheritemJson.put("def9", InvoiceConstant.INVOICE_BUSINESS_FORMAT_001); //业态，必选，看云文档业态
//        ar_gatheritemJson.put("def11", ""); //转出前房号编码，收除了诚意金之外的首期等需要选
        // TODO: 2024/11/14 具体编码应该是根据小区来的
        ar_gatheritemJson.put("def10", InvoiceConstant.INVOICE_MATERIAL_djjy); //项目档案编码，看云文档项目档案对照
        ar_gatheritemJson.put("top_billid", pkInformerId); //到账通知生单时,（银行流水中返回的id，见5.2.10到账通知认领查询pk_informer字段）
        ar_gatheritem.add(ar_gatheritemJson);
        json.put("ar_gatheritem", ar_gatheritem);

        try {
            String result = InvoiceUtil.gatheringbillAdd(json);
            JSONObject jsonObject = JSONObject.parseObject(result);
            if (jsonObject.getString("success").equals("true")) {
                roomSelectionInfo.setNcJson(result);
                updateByPrimaryKeySelective(roomSelectionInfo);
            }

        } catch (Exception e) {
            log.error("收定金凭证推送报错", e);
            return AjaxResult.error(e.getMessage());
        }
        return AjaxResult.success();
    }


}
