package com.idea.invoice.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.idea.buildManage.mapper.ParkInfoMapper;
import com.idea.buildManage.mapper.ParkRoomExtendMapper;
import com.idea.buildManage.mapper.ParkRoomMapper;
import com.idea.buildManage.model.ParkFloorDisc;
import com.idea.buildManage.model.ParkInfo;
import com.idea.buildManage.model.ParkRoom;
import com.idea.buildManage.response.ParkRoomResponse;
import com.idea.customerManagement.dto.ContractManageDto;
import com.idea.customerManagement.mapper.BuyerMapper;
import com.idea.customerManagement.mapper.CustomerManagementMapper;
import com.idea.customerManagement.model.Buyer;
import com.idea.customerManagement.model.BuyerExample;
import com.idea.customerManagement.model.ContractManage;
import com.idea.customerManagement.model.CustomerManagement;
import com.idea.customerManagement.service.BuyerService;
import com.idea.customerManagement.service.ContractManageService;
import com.idea.invoice.dto.InvoiceManageDto;
import com.idea.invoice.mapper.InvoiceManageExtendMapper;
import com.idea.invoice.mapper.InvoiceManageMapper;
import com.idea.invoice.model.*;
import com.idea.invoice.util.InvoiceConstant;
import com.idea.invoice.util.InvoiceUtil;
import com.idea.paymentManagement.model.PayLog;
import com.idea.paymentManagement.model.PayLogExample;
import com.idea.paymentManagement.service.PayLogService;
import com.idea.util.DateUtils;
import com.rockstar.common.base.BaseService;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.model.extend.DateTrans;
import com.rockstar.frame.model.extend.Tablepar;
import com.rockstar.shiro.util.ShiroUtils;
import com.rockstar.system.service.SysDictService;
import com.rockstar.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class InvoiceManageService implements BaseService<InvoiceManage, InvoiceManageExample> {

    @Autowired
    private InvoiceManageMapper invoiceManageMapper;
    @Autowired
    private InvoiceManageExtendMapper extendMapper;
    @Autowired
    private PayLogService payLogService;
    @Autowired
    private ContractManageService contractManageService;
    @Autowired
    private BuyerService buyerService;
    @Autowired
    private ParkRoomExtendMapper roomExtendMapper;
    @Autowired
    private ParkRoomMapper parkRoomMapper;
    @Autowired
    private InvoiceBankService invoiceBankService;
    @Autowired
    private SysDictService sysDictService;
    // 注入自身 防止事务不生效
    @Autowired
    private InvoiceManageService invoiceManageService;
    @Autowired
    private CustomerManagementMapper customerManagementMapper;
    @Autowired
    private ReceiptManageService receiptManageService;
    @Autowired
    private ParkInfoMapper parkInfoMapper;
    @Autowired
    private BuyerMapper buyerMapper;


    @Override
    public int deleteByPrimaryKey(String s) {
        return invoiceManageMapper.deleteByPrimaryKey(s);
    }

    @Override
    public int insertSelective(InvoiceManage invoiceManage) {
        invoiceManage.setId(IdUtil.simpleUUID());
        return invoiceManageMapper.insertSelective(invoiceManage);
    }

    public int insertWithoutId(InvoiceManage invoiceManage) {
        return invoiceManageMapper.insertSelective(invoiceManage);
    }

    @Override
    public InvoiceManage selectByPrimaryKey(String s) {
        return invoiceManageMapper.selectByPrimaryKey(s);
    }

    @Override
    public int updateByPrimaryKeySelective(InvoiceManage invoiceManage) {
        return invoiceManageMapper.updateByPrimaryKeySelective(invoiceManage);
    }

    @Override
    public int updateByExampleSelective(InvoiceManage invoiceManage, InvoiceManageExample invoiceManageExample) {
        return invoiceManageMapper.updateByExampleSelective(invoiceManage, invoiceManageExample);
    }

    @Override
    public int updateByExample(InvoiceManage invoiceManage, InvoiceManageExample invoiceManageExample) {
        return invoiceManageMapper.updateByExample(invoiceManage, invoiceManageExample);
    }

    @Override
    public List<InvoiceManage> selectByExample(InvoiceManageExample invoiceManageExample) {
        return invoiceManageMapper.selectByExample(invoiceManageExample);
    }

    @Override
    public long countByExample(InvoiceManageExample invoiceManageExample) {
        return invoiceManageMapper.countByExample(invoiceManageExample);
    }

    @Override
    public int deleteByExample(InvoiceManageExample invoiceManageExample) {
        return invoiceManageMapper.deleteByExample(invoiceManageExample);
    }


    /**
     * 预收款发票
     *
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    public PageInfo<InvoiceManageDto> advancePaymentInvoiceList(Tablepar tablepar, InvoiceManageDto model, DateTrans dt) {
        model.setType("1");
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<InvoiceManageDto> list = extendMapper.listByModel(model);
        for (InvoiceManageDto invoiceManage : list) {
//            String money = payLogService.getMoneyByContractId(invoiceManage.getContractId(), null);
//            invoiceManage.setReceivedMoney(new BigDecimal(money));
            // 预收款发票 已收金额
            invoiceManage.setReceivedMoney(invoiceManage.getInvoiceAmount());
        }
        return new PageInfo<>(list);
    }

    /**
     * 新增预收款发票
     *
     * @param model
     * @return
     */
    public int add(InvoiceManageDto model) {

        String id = IdUtil.simpleUUID();
        model.setId(id);
        model.setInvoiceStatus("0");
        model.setCreateTime(new Date());
        model.setCreateUser(ShiroUtils.getUserId());
        model.setSendStatus("0");
        int result = insertWithoutId(model);

        // 预收款发票 新增后 推送用友
        if (model.getType().equals("1")) {
            try {
                String resultJson = sendInvoice(model);
                JSONObject jsonObject = JSONObject.parseObject(resultJson);
                String success = jsonObject.getString("success");
                if ("true".equals(success)) {
                    String code = jsonObject.getJSONObject("data").getJSONObject("ar_recbill").getString("code");
                    model.setSendStatus("1");
                    model.setSubmitTime(new Date());
                    model.setSubmitUser(ShiroUtils.getUserId());
                    model.setNcJson(resultJson);
                    model.setNcCode(code);
                    updateByPrimaryKeySelective(model);
                } else if ("false".equals(success)) {
                    throw new Exception(jsonObject.getString("message"));
                }
            }catch (Exception e){
                log.error("推送预收款发票失败", e);
            }
        }
        return result;
    }


    public InvoiceManageDto getById(String id) {
        InvoiceManage invoiceManage = selectByPrimaryKey(id);
        InvoiceManageDto result = new InvoiceManageDto();
        BeanUtil.copyProperties(invoiceManage, result);
        return result;
    }

    /**
     * 正式发票管理
     *
     * @param tablepar
     * @param model
     * @param dt
     * @return
     */
    public PageInfo<InvoiceManageDto> formalInvoiceList(Tablepar tablepar, InvoiceManageDto model, DateTrans dt) {
        model.setType("2");
        PageHelper.startPage(tablepar.getPageNum(), tablepar.getPageSize());
        List<InvoiceManageDto> list = extendMapper.listByModel(model);
        for (InvoiceManageDto invoiceManage : list) {
            String money = "";
            // 房款
            if (invoiceManage.getPaymentType().equals("1")) {
                money = payLogService.getMoneyByContractId(invoiceManage.getContractId(), 1);
                invoiceManage.setReceivableMoney(invoiceManage.getBuyerMoney());
            }
            // 专项维修资金
            if (invoiceManage.getPaymentType().equals("2")) {
                money = payLogService.getMoneyByContractId(invoiceManage.getContractId(), 2);
                invoiceManage.setReceivableMoney(invoiceManage.getMaintenanceTotalPrice());
            }
            invoiceManage.setReceivedMoney(new BigDecimal(money));
        }
        return new PageInfo<>(list);
    }

    /**
     * 正式发票管理 全部数据
     * @param model
     * @param dt
     * @return
     */
    public List<InvoiceManageDto> formalInvoiceListAll(InvoiceManageDto model, DateTrans dt) {
        model.setType("2");
        List<InvoiceManageDto> list = extendMapper.listByModel(model);
        for (InvoiceManageDto invoiceManage : list) {
            String money = "";
            // 房款
            if (invoiceManage.getPaymentType().equals("1")) {
                money = payLogService.getMoneyByContractId(invoiceManage.getContractId(), 1);
                invoiceManage.setReceivableMoney(invoiceManage.getBuyerMoney());
            }
            // 专项维修资金
            if (invoiceManage.getPaymentType().equals("2")) {
                money = payLogService.getMoneyByContractId(invoiceManage.getContractId(), 2);
                invoiceManage.setReceivableMoney(invoiceManage.getMaintenanceTotalPrice());
            }
            invoiceManage.setReceivedMoney(new BigDecimal(money));
        }
        return list;
    }


    /**
     * 根据合同id查询
     *
     * @param contractId
     * @return
     */
    public InvoiceManageDto getInfoByContract(String contractId, String invoiceGoodsName, Integer payType) {

        InvoiceManageDto invoiceManage = new InvoiceManageDto();
        // 合同信息
        ContractManageDto contract = contractManageService.getById(contractId);
        // 买受人信息
        BuyerExample buyerExample = new BuyerExample();
        buyerExample.setOrderByClause("order_num asc");
        buyerExample.createCriteria().andCustomerManagementIdEqualTo(contract.getCustomerManagementId());
        List<Buyer> buyers = buyerService.selectByExample(buyerExample);
        Buyer buyer = buyers.get(0);
        // 房间信息
        ParkRoomResponse room = roomExtendMapper.getFullName(contract.getHouseId());
        StringBuilder builder = new StringBuilder();
        for (Buyer item : buyers) {
            if (item.getRelationship() != null) {
                builder.append(item.getName()).append("身份证号：").append(item.getIdentityCard()).append("\n");
            }
        }
        PayLogExample payLogExample = new PayLogExample();
        PayLogExample.Criteria criteria = payLogExample.createCriteria();
        criteria.andContractIdEqualTo(contractId)
                .andStatusEqualTo(0);
        if (ObjectUtil.isNotNull(payType)) {
            criteria.andPayTypeEqualTo(payType);
        }
        List<PayLog> payLogs = payLogService.selectByExample(payLogExample);
        BigDecimal money = BigDecimal.ZERO;
        for (PayLog payLog : payLogs) {
            money = money.add(payLog.getPayMoney());
        }

        invoiceManage.setContractId(contractId);
        invoiceManage.setCustomerManagementId(contract.getCustomerManagementId());
        invoiceManage.setContractNumber(contract.getContractNumber());
        invoiceManage.setRecordNumber(contract.getRecordNumber());
        invoiceManage.setInvoiceHeaderName(buyer.getName());
        invoiceManage.setInvoiceHeaderNumber(buyer.getIdentityCard());
        // TODO: 2024/10/17 商品编码目前为空 对接接口后调整
        invoiceManage.setInvoiceGoodsCode(InvoiceConstant.INVOICE_GOODS_CODE_ADVANCE_PAYMENT);
        invoiceManage.setInvoiceGoodsName(invoiceGoodsName + room.getGroupName());
        invoiceManage.setSpecifications(room.getBuildName() + "-" + room.getRoomNo());
        invoiceManage.setInvoiceCount(new BigDecimal(contract.getActualBuildArea().toString()));
        invoiceManage.setInvoiceAmount(money);
        invoiceManage.setInvoiceTax("0");
        invoiceManage.setRemark(builder.toString());
        return invoiceManage;
    }


    /**
     * 查询对应的银行
     *
     * @param id
     * @return
     */
    public JSONObject getBankTaxRateByFloor(String id) {
        JSONObject jsonObject = new JSONObject();
        String result = "";
        InvoiceManage invoiceManage = selectByPrimaryKey(id);
        // 1-房款，2-专项维修资金
        String paymentType = invoiceManage.getPaymentType();
        String dictLabel = sysDictService.getDictLabel("PAYMENT_TYPE", paymentType);
        InvoiceBankExample example = new InvoiceBankExample();
        example.createCriteria().andSuitCostTypeLike("%" + dictLabel + "%");
        List<InvoiceBank> bankList = invoiceBankService.selectByExample(example);
        if (CollectionUtils.isNotEmpty(bankList)) {
            ContractManage parkContractManage = contractManageService.selectByPrimaryKey(invoiceManage.getContractId());
            ParkRoom parkRoom = parkRoomMapper.selectByPrimaryKey(parkContractManage.getHouseId());
            if (StringUtils.isNotEmpty(parkRoom.getDiscId())) {
                for (InvoiceBank invoiceBank : bankList) {
                    if (StringUtils.isNotEmpty(invoiceBank.getSuitFloor())) {
                        String[] floors = invoiceBank.getSuitFloor().split(",");
                        List<String> suitCostTypes = Arrays.asList(invoiceBank.getSuitCostType().split(","));
                        List<String> taxRates = Arrays.asList(invoiceBank.getTaxRate().split(","));
                        for (String floor : floors) {
                            if (parkRoom.getDiscId().contains(floor)) {
                                result = invoiceBank.getBankName() + invoiceBank.getBankBranchName() + invoiceBank.getBankNumber();
                                jsonObject.put("invoiceSellerBank", result);
                                int index = suitCostTypes.indexOf(dictLabel);
                                String taxRate = taxRates.get(index);
                                jsonObject.put("invoiceTax", taxRate);
                                break;
                            }
                        }
                    }
                }
            }

        }

        return jsonObject;
    }

    /**
     * @param contractId
     * @param paymentType 1-房款，2-专项维修资金
     * @return
     */
    public JSONObject getBankTaxRateByFloor(String contractId, String paymentType) {
        JSONObject jsonObject = new JSONObject();
        String result = "";
        String dictLabel = sysDictService.getDictLabel("PAYMENT_TYPE", paymentType);
        InvoiceBankExample example = new InvoiceBankExample();
        example.createCriteria().andSuitCostTypeLike("%" + dictLabel + "%");
        List<InvoiceBank> bankList = invoiceBankService.selectByExample(example);
        if (CollectionUtils.isNotEmpty(bankList)) {
            ContractManage parkContractManage = contractManageService.selectByPrimaryKey(contractId);
            ParkRoom parkRoom = parkRoomMapper.selectByPrimaryKey(parkContractManage.getHouseId());
            if (StringUtils.isNotEmpty(parkRoom.getDiscId())) {
                for (InvoiceBank invoiceBank : bankList) {
                    if (StringUtils.isNotEmpty(invoiceBank.getSuitFloor())) {
                        String[] floors = invoiceBank.getSuitFloor().split(",");
                        for (String floor : floors) {
                            if (parkRoom.getDiscId().contains(floor)) {
                                result = invoiceBank.getBankName() + invoiceBank.getBankBranchName() + invoiceBank.getBankNumber();
                                jsonObject.put("invoiceSellerBank", result);
                                // 预收款发票税率默认为0
                                jsonObject.put("invoiceTax", "0");
                                break;
                            }
                        }
                    }
                }
            }

        }

        return jsonObject;
    }


    /**
     * 批量开票列表
     *
     * @param invoiceManages
     * @return
     */
    public List<InvoiceManageDto> batchListVo(String invoiceManages) {

        List<InvoiceManageDto> result = new ArrayList<>();
        String[] ids = invoiceManages.split(",");
        for (String id : ids) {
            InvoiceManageDto model = getById(id);
            JSONObject jsonObject = getBankTaxRateByFloor(id);
            model.setInvoiceSellerBank(jsonObject.getString("invoiceSellerBank"));
            model.setInvoiceTax(jsonObject.getString("invoiceTax"));
            result.add(model);
        }
        return result;
    }

    /**
     * 推送开票
     *
     * @param model
     */
    public String sendInvoice(InvoiceManageDto model) throws Exception {

        // 查询客户管理信息
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(model.getCustomerManagementId());
        // 查询小区信息
        ParkInfo parkInfo = parkInfoMapper.selectByPrimaryKey(customerManagement.getGroupId());
        // 查询房间信息
        ParkRoom parkRoom = parkRoomMapper.selectByPrimaryKey(customerManagement.getHouseId());

        String address = parkInfo.getAddress();
        Double invoiceTax = Double.parseDouble(model.getInvoiceTax()) * 100;

        JSONObject json = new JSONObject();

        JSONObject ar_recbill = new JSONObject();
        String srcsystemid = model.getId();
        String billdate = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD_HH_MM_SS, new Date());
        ar_recbill.put("srcsystemid", srcsystemid); //外系统标识,必选
        ar_recbill.put("pk_tradetype", InvoiceConstant.INVOICE_BUSINESS_F0_Cxx_05); //交易类型,必选
        ar_recbill.put("pk_org", InvoiceUtil.pk_org); //组织编码,必选
        ar_recbill.put("billdate", billdate); //单据日期
        ar_recbill.put("def2", srcsystemid); //外系统单据号，必填，
        ar_recbill.put("def19", "E05"); //特殊票种标识，默认是E05
        ar_recbill.put("def20", "0"); //征税方式
        ar_recbill.put("def21", address); //不动产地址，必填，字符串
        ar_recbill.put("def22", "N"); //是否跨地址标志,必填，Y-是，N-否
        json.put("ar_recbill", ar_recbill);

        JSONArray ar_recitem = new JSONArray();
        JSONObject ar_recitemJson = new JSONObject();
        ar_recitemJson.put("srcsystemid", srcsystemid); //外系统子表标识
        ar_recitemJson.put("scomment", ""); //摘要，非必选
        ar_recitemJson.put("objtype", "0"); //0=客户，1=供应商--判断customer和supplier是否必填，必选
        ar_recitemJson.put("customer", model.getInvoiceHeaderNumber()); //客商的纳税人识别号
        ar_recitemJson.put("money_de", model.getInvoiceAmount()); //金额，必选
        ar_recitemJson.put("taxrate", invoiceTax); //税率，必选，无税传0
        ar_recitemJson.put("def11", parkRoom.getId()); //转出前房号编码，收除了诚意金之外的首期等需要选
        ar_recitemJson.put("def9", InvoiceConstant.INVOICE_BUSINESS_FORMAT_001); //业态，必选，看云文档业态
        // TODO: 2024/11/6 没有 
        ar_recitemJson.put("def7", ""); //房产工程项目，必选，看云文档
        ar_recitemJson.put("def8", parkRoom.getBuildId()); //楼栋，必选，看云文档
        ar_recitemJson.put("material", InvoiceConstant.INVOICE_MATERIAL_hjjy); //物料
        ar_recitem.add(ar_recitemJson);
        json.put("ar_recitem", ar_recitem);

        String result = InvoiceUtil.recbillAdd(json);

        return result;

    }

    /**
     * 批量推送
     *
     * @param list
     * @return
     */
    public AjaxResult batchSendInvoice(List<InvoiceManageDto> list) {

        // 推送前校验状态
        for (InvoiceManageDto model : list) {
            InvoiceManage invoiceManage = selectByPrimaryKey(model.getId());
            if (null != invoiceManage && !InvoiceConstant.INVOICE_STATUS_WAIT.equals(invoiceManage.getInvoiceStatus())
                    && !InvoiceConstant.INVOICE_STATUS_ERROR.equals(invoiceManage.getInvoiceStatus())) {
                return AjaxResult.error(504, "提交失败，开票状态发生变化");
            }
        }
        try {
            for (InvoiceManageDto invoiceManage : list) {
                String result = invoiceManageService.sendInvoice(invoiceManage);
                JSONObject jsonObject = JSONObject.parseObject(result);
                String success = jsonObject.getString("success");
                if ("true".equals(success)) {
                    String code = jsonObject.getJSONObject("data").getJSONObject("ar_recbill").getString("code");
                    invoiceManage.setSendStatus("1");
                    invoiceManage.setSubmitTime(new Date());
                    invoiceManage.setSubmitUser(ShiroUtils.getUserId());
                    invoiceManage.setNcJson(result);
                    invoiceManage.setNcCode(code);
                    invoiceManageService.updateByPrimaryKeySelective(invoiceManage);
                } else if ("false".equals(success)) {
                    throw new RuntimeException(jsonObject.getString("message"));
                }
            }
        }catch (Exception e){
            log.error("批量推送发票异常",e);
            return AjaxResult.error(504, e.getMessage());
        }

        return AjaxResult.success();
    }

    /**
     * 票据管理 新增记录 开票后调用
     *
     * @param model
     * @return
     */
    public int addReceiptManage(InvoiceManage model) {
        // 票据管理 新增一条记录
        ContractManage contractManage = contractManageService.selectByPrimaryKey(model.getContractId());
        CustomerManagement customerManagement = customerManagementMapper.selectByPrimaryKey(contractManage.getCustomerManagementId());
        ReceiptManage receiptManage = new ReceiptManage();
        receiptManage.setBatchNumber(customerManagement.getBatchNumber());
        receiptManage.setRoomSelectionNumber(customerManagement.getRoomSelectionNumber());
        receiptManage.setBuyerName(customerManagement.getBuyerName());
        receiptManage.setCustomerManagementId(model.getCustomerManagementId());
        // 票据类型
        // 预收款发票
        if (model.getType().equals("1")) {
            receiptManage.setReceiptType("5");
        }
        // 正式发票
        if (model.getType().equals("2")) {
            receiptManage.setReceiptType("6");
        }
        receiptManage.setReceiptMoney(model.getInvoiceAmount());
        receiptManage.setReceiptNumber(model.getInvoiceNo());
        int result = receiptManageService.add(receiptManage,ShiroUtils.getUserId());
        return result;
    }

    /**
     * 查询需要更新状态的发票
     *
     * @return
     */
    public List<InvoiceManage> selectNeedRefreshStatusList() {
        return extendMapper.selectNeedRefreshStatusList();
    }

}
