package com.idea.pro.wx.web;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import com.rockstar.biz.model.BizMain;
import com.rockstar.biz.model.BizUser;
import com.rockstar.biz.service.BizMainService;
import com.rockstar.common.domain.AjaxResult;
import com.rockstar.frame.mapper.extend.BizMainDao;
import com.rockstar.frame.model.FrameUser;
import com.rockstar.frame.service.FrameUserService;
import com.rockstar.h5.service.FrameWxService;
import com.rockstar.user.model.UserAccount;
import com.rockstar.user.model.UserMain;
import com.rockstar.user.service.UserAccountService;
import com.rockstar.user.service.UserMainService;
import com.rockstar.util.*;
import com.rockstar.wx.base.BaseWxController;
import com.rockstar.wx.dto.UserInfo;
import com.rockstar.wx.dto.WxLoginInfo;
import com.rockstar.wx.pojo.LoginRequest;
import com.rockstar.wx.service.UserTokenManager;
import com.rockstar.wx.util.EmojiUtil;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.cache.CacheManager;
import org.apache.shiro.mgt.RealmSecurityManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 鉴权服务
 */
@RestController
@RequestMapping("/wx/auth")
@Validated
public class WxAuthController extends BaseWxController {
    private final Log logger = LogFactory.getLog(WxAuthController.class);
    private final String sign = "AIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDz2fCOYaaU6sztFql4cOmiFRq2";

    @Autowired
    private WxMaService wxService;

    @Autowired
    private UserAccountService accountService;

    @Autowired
    private UserMainService userMainService;


    @Autowired
    private BizMainDao bizMainDao;

    @Autowired
    private BizMainService bizMainService;

    @Autowired
    private FrameWxService frameWxService;

    @Autowired
    private FrameUserService frameUserService;


    /**
     * 微信登录
     *
     * @return 登录结果
     */
    @PostMapping("login_by_weixin")
    @ResponseBody
    public Object loginByWeixin(@RequestBody WxLoginInfo wxLoginInfo,HttpServletRequest request) throws InvocationTargetException, IllegalAccessException, WxErrorException {
        //获取用户code
        String code = wxLoginInfo.getCode();
        UserInfo userInfo = wxLoginInfo.getUserInfo();
        Integer type = wxLoginInfo.getType();
//        String encryptedData = wxLoginInfo.getEncryptedData();
//        String iv = wxLoginInfo.getIv();
        if (code == null || userInfo == null) {
            return ResponseUtil.badArgument();
        }
        String sessionKey = null;
        String openId = null;
        String uid = null;
        String unionid = null;
//        String bank = "";
        try {
            WxMaJscode2SessionResult result = this.wxService.getUserService().getSessionInfo(code);
            openId = result.getOpenid();
            sessionKey = result.getSessionKey();
            unionid = result.getUnionid();
        } catch (Exception e) {
            e.printStackTrace();
        }

        UserAccount account = null;
        UserMain user = null;
        if(StringUtils.isNotEmpty(openId)){
            account = accountService.getAccountByOpenId(openId);
        }

        if (account == null) {
            user = new UserMain();
            uid = SnowflakeIdWorker.getUUID();
            user.setId(uid);
            user.setNickName(userInfo.getNickName());
            user.setGender(String.valueOf(userInfo.getGender()));
            user.setCreatedAt(new Date());
            userMainService.insertWithoutId(user);

            account = new UserAccount();
            BeanUtils.copyBeanProp(account, userInfo);
            account.setUserId(uid);
            account.setOpenId(openId);
            account.setAccount(UUIDUtils.shortUUID().toUpperCase());
            account.setPassword(MD5Util.encode(account.getAccount()));
            account.setCreatedAt(new Date());
            accountService.insert(account);

        } else {
            uid = account.getUserId();
            user = userMainService.selectByPrimaryKey(account.getUserId());
            userInfo.setNickName(user.getNickName());
            userInfo.setAvatarUrl(user.getAvatarUrl());
        }
        if (sessionKey == null || openId == null) {
            return ResponseUtil.fail();
        }
        String nickname = userInfo.getNickName();
        userInfo.setNickName(EmojiUtil.emojiConverterToAlias(nickname));
        String token = UserTokenManager.generateToken(uid);
        Map<Object, Object> result = new HashMap<Object, Object>();
        result.put("token", token);
        result.put("userInfo", userInfo);
        result.put("phone", user.getPhoneNo());
        result.put("uid", uid);
        return ResponseUtil.ok(result);
    }

    /**
     * 用户登录 - H5及web登录，根据登录类型（loginRequest.getType()）来判断是平台端还是商家端
     *
     * @return 登录结果
     */
    @RequestMapping(value="/login_by_app_main", method= RequestMethod.POST,produces = {MediaType.APPLICATION_JSON_VALUE})
    @ResponseBody
    public Object loginByMain(LoginRequest loginRequest) {
        return login(loginRequest);
    }

    public Object login(LoginRequest loginRequest) {
        String userName = loginRequest.getUserName();
        String rawPassword = loginRequest.getPassword();

        String type = loginRequest.getType(); //用户类型，USER-终端用户，FRAME-管理用户，BIZ-商户用户 BIZPERSON-商户用户个体户
        String mainType = ""; //用户类型，USER-终端用户，FRAME-管理用户，BIZ-商户用户 BIZPERSON-商户用户个体户
        if("BIZ_PERSON".equals(type)){
            loginRequest.setType("BIZ");
            mainType = Constant.USER_RELATION_STATUS_1;
        }else if("BIZ".equals(type)){
            mainType = Constant.USER_RELATION_STATUS_0;
        }


        if (StringUtils.isEmpty(userName)) {
            return ResponseUtil.badArgument();
        }

        if(StringUtils.isNotEmpty(loginRequest.getCaptchaId())){
            if (StringUtils.isBlank(loginRequest.getCode())) {
                return ResponseUtil.fail(507, "请输入验证码");
            }

            String validCode = HomeCacheManager.getCaptcha(loginRequest.getCaptchaId());
            if(StringUtils.isEmpty(validCode)){
                return ResponseUtil.fail(507, "无法获取验证码信息");
            }

            if(!StringUtils.equals(validCode,loginRequest.getCode())){
                return ResponseUtil.fail(507, "验证码不匹配");
            }
        }
        Object user = frameWxService.queryUserName(userName,loginRequest.getType());

        if(null==user){
            return ResponseUtil.fail(507, "无此用户");
        }

        String password = "";
        String uid = "";
        BizMain biz = null;
        if(user instanceof FrameUser){
            password = ((FrameUser) user).getPassword();
            uid = ((FrameUser) user).getId();
            ((FrameUser) user).setPassword("");

            if(!password.equals(DigestUtils.md5Hex(DigestUtils.md5Hex(rawPassword)))){
                return ResponseUtil.fail(508, "用户密码错误");
            }

            if("0".equals(((FrameUser) user).getStatus())){
                return ResponseUtil.fail(508, "该用户已被禁用");
            }
        }else if(user instanceof BizUser){
            password = ((BizUser) user).getPassword();
            uid = ((BizUser) user).getId();
            biz = bizMainDao.selectByPrimaryKey(((BizUser) user).getMainId());
            BizMain bizMain = bizMainService.selectByPrimaryKey(((BizUser) user).getMainId());
            ((BizUser) user).setPassword("");

            if(!password.equals(DigestUtils.md5Hex(rawPassword))){
                return ResponseUtil.fail(508, "用户密码错误");
            }

            if("1".equals(biz.getStatus())){
                return ResponseUtil.fail(508, "该商户已被禁用");
            }

            if(!mainType.equals(bizMain.getType())){
                return ResponseUtil.fail(508, "用户登录体系与信息不一致");
            }
        }
//
//        if(!password.equals(DigestUtils.md5Hex(rawPassword))){
//            return ResponseUtil.fail(508, "用户密码错误");
//        }

        String token = UserTokenManager.generateToken(uid);
        Map<Object, Object> result = new HashMap<Object, Object>();
        result.put("token", token);
        result.put("user", user);
        result.put("biz", biz);
        result.put("uid", uid);
        HomeCacheManager.removeCaptcha(loginRequest.getCaptchaId());
        return ResponseUtil.ok(result);
    }

    /**
     * 本地获取token
     */
    @PostMapping("getToken/")
    public Object getToken(@RequestBody String body, HttpServletRequest request) {
        return gotToken(body, this.sign, request);
    }

    @RequestMapping(value="/adminUnlock", method= RequestMethod.POST,produces = {MediaType.APPLICATION_JSON_VALUE})
    @ResponseBody
    public AjaxResult adminUnlock() {
        FrameUser user = frameUserService.selectByPrimaryKey("1");
        if (null != user && Constant.TRUE.equals(user.getIsLock())) {
            user.setIsLock(Constant.FALSE);
            RealmSecurityManager rsm = (RealmSecurityManager) SecurityUtils.getSecurityManager();
            CacheManager cache = rsm.getCacheManager();
            if (null != cache.getCache("passwordRetryCache")) {
                cache.getCache("passwordRetryCache").remove(user.getUsername());
            }
            frameUserService.updateByPrimaryKeySelective(user);
        }
        return AjaxResult.success();
    }


}
